uniform float g_Time;

#define USE_NOISE 1

#ifndef TIME_BIAS
#define TIME_BIAS 0.0
#endif

uniform vec4 m_Color;
uniform sampler2D m_ColorMap;
uniform sampler2D m_ColorMap2;
uniform sampler2D m_ColorMap3;
uniform sampler2D m_Noise;
uniform sampler2D m_Lookups;

varying vec2 texCoord1;
varying vec2 texCoord2;
varying vec4 vColor;

const float speed = 1.0; 
const float flySpeed = FLY_SPEED; //1.5; 
const float flyLength = FLY_LENGTH; //0.5;

vec2 textureNoise( in vec2 texCoord, in float time ) {
    vec2 result = texCoord;
    vec2 noiseCoord = texCoord;

    noiseCoord.y = noiseCoord.y - time;
    
    vec4 noise = texture2D(m_Noise, noiseCoord);
    result.x += noise.r * 0.2 - 0.1;     

    noiseCoord.x = noiseCoord.x + (time*0.8);
    noise = texture2D(m_Noise, noiseCoord);
    result.y += noise.r * 0.1 - 0.05;     
    
    return result;
}

float calculateAlpha( in float offset )
{
    float d = offset * 2.0 - 1.0;
    return 1.0 - (d * d);
}

void main(){
    vec4 color = vColor; 

    vec2 tc = texCoord2;
    tc.y = 1.0 - ((1.0 - tc.y) * (1.0 - tc.y));


    // Calculate values for texture 1 
    #ifdef USE_NOISE   
    vec2 noiseCoord = textureNoise( tc, (g_Time+TIME_BIAS) * speed );
    #else
    vec2 noiseCoord = tc; 
    #endif    
 
    float offset1 = mod((g_Time+TIME_BIAS) * flySpeed, flyLength)/flyLength;
    noiseCoord.y -= (offset1 * offset1) * flyLength; 
    vec4 texel = texture2D( m_ColorMap, noiseCoord );

    float alpha1 = clamp(calculateAlpha(offset1) * 1.0, 0.0, 1.0);
    alpha1 *= clamp(tc.y * tc.y * 4.0, 0.0, 1.0);



    // Calculate values for texture 2 
    #ifdef USE_NOISE   
    noiseCoord = textureNoise( tc, (g_Time+TIME_BIAS) * speed * 0.9 );
    #else
    noiseCoord = tc; 
    #endif
    
    float offset2 = mod((g_Time+TIME_BIAS) * flySpeed + flyLength * 0.333, flyLength)/flyLength;   
    noiseCoord.y -= (offset2 * offset2) * flyLength; 
    vec4 texel2 = texture2D( m_ColorMap2, noiseCoord );

    float alpha2 = clamp(calculateAlpha(offset2) * 1.0, 0.0, 1.0);
    alpha2 *= clamp(tc.y * tc.y * 4.0, 0.0, 1.0);


    // Calculate values for texture 3 
    #ifdef USE_NOISE   
    noiseCoord = textureNoise( tc, (g_Time+TIME_BIAS) * speed * 0.7 );
    #else
    noiseCoord = tc;
    #endif
    
    float offset3 = mod((g_Time+TIME_BIAS) * flySpeed + flyLength * 0.666, flyLength)/flyLength;   
    noiseCoord.y -= (offset3 * offset3) * flyLength; 
    vec4 texel3 = texture2D( m_ColorMap3, noiseCoord );

    float alpha3 = clamp(calculateAlpha(offset3) * 1.0, 0.0, 1.0);
    alpha3 *= clamp(tc.y * tc.y * 4.0, 0.0, 1.0);


    // Using noise for a heat band effect but it ultimately seems
    // unncessary and is incompatible with the latest blending anyway 
    //noiseCoord.x = tc.x * 0.1;
    //noiseCoord.y = noiseCoord.y - g_Time * speed * 0.5;
    //vec4 noise = texture2D(m_Noise, noiseCoord);
    //noise.x *= 2;    
    //color = (texel * (2.0-noise.x)) * alpha1 + (texel2 * noise.x) * alpha2 + (texel3 * alpha3);
    
 
    #ifdef BLEND_ADD
    // Regular add blending   
    color = texel * alpha1 + texel2 * alpha2 + texel3 * alpha3;
    #else    
    // Pur mix blending
    //color = mix(texel * alpha1, texel2 * alpha2, 0.5);
    //color = mix(color, texel3 * alpha3, 0.5);

    // A weighted averaging
    color = texel * alpha1 + texel2 * alpha2 + texel3 * alpha3;
    color *= 1.0/(alpha1 + alpha2 + alpha3);
    #endif 
 
    // Accumulated alpha value
    color.a = color.r + color.g + color.b;
 
    // Fade at the edges   
    float xOffset = -1.0 + tc.x * 2.0; 
    float fade = (1.0 - (xOffset * xOffset)) * (1.0 - tc.y);
    color.a = color.a * fade; 
 
    #ifdef ROOT_GLOW
    float xDiff = clamp((tc.x - 0.5) * 3.0, -1.0, 1.0);
    float yDiff = clamp((tc.y - 0.5) * 2.0, -1.0, 1.0); 
    float fade2 = clamp(1.0 - (xDiff * xDiff + yDiff * yDiff), 0.0, 1.0);
    color.r += fade2 * 0.5; // * (1.0-color.a);
    //color.g += fade2 * fade2 * 0.8 * 0.5; // * (1.0-color.a);
    color.a += fade2 * 0.5;
    // Setting the fade directly creates a more fake looking fire that
    // might be worth pursuing for a "painterly" look
    //color.a = fade2 * 0.5;
    #endif
    
    /*
    actually, I can no longer duplicate the effect, oh well
    //Unclamped like this it creates a seriously cool effect...
    //that looks nothing like real fire.
    float xDiff = clamp((tc.x - 0.5) * 5.0, -1.0, 1.0);
    float yDiff = clamp((tc.y - 0.5) * 4.0, -1.0, 1.0); 
    float fade2 = 1.0 - (xDiff * xDiff + yDiff * yDiff); //clamp(1.0 - (xDiff * xDiff + yDiff * yDiff), 0.0, 1.0);
    //color.a = color.r + color.g + color.b; 
    color.r += fade2 * (1.0-color.a);
    color.g += fade2 * fade2 * 0.8 * (1.0-color.a);
    //color.a = color.a * fade + fade2 * 0.5;
    color.a += fade2 * 0.5;
    */

    gl_FragColor = color;    
}
